import numpy as np
import ipywidgets as widgets

def plot_maclaurin_polynomials(ax, f, X, derivative, title, start=None, stop=None, step=None, degree=None):
    '''
    Plot the Maclaurin polynomials of the function f
    with degrees from start (inclusive) to stop (exclusive) by step.
    
    ax
      Diagram.
    f
      Real function that operates also element-wise on ndarrays.
    X : ndarray
      Input values for the function f.
    derivative(n : integer) : float
      Returns the n-the derivative of the function f in 0.
    start : integer
    stop : integer
    step : integer
    degree : integer
      Initial degree of the Maclaurin polynomial for the function f.
    '''

    if start is None:
        start = 0

    if stop is None:
        stop = 5

    if step is None:
        step = 1

    if degree is None:
        degree = (stop - 1 - start) // 2

    ## Maclaurin polynomials
    def T(n, X): # Maclaurin polynomial of the n-th degree
        if n >= 0:
            return T(n-1, X) + derivative(n) * X ** n / np.math.factorial(n)
        else:
            return np.zeros(len(X))

    ## graf of the function
    color = ax.plot([], [], label="funkcia")[0].get_color()
    ax.plot(X, f(X), c=color)
    ax.plot(0, f(0), 'o', c=color)

    ## graf of the Maclaurin polynomial
    color = ax.plot([], [], label="polynóm")[0].get_color()
    zorder = len(ax.lines)

    def plot_polynom(n):
        ax.set_title(title.format(n), fontdict={'verticalalignment': 'bottom'})
        if len(ax.lines) > zorder: # update value
            ax.lines[zorder].set_ydata(T(n, X))
        else: # initial value
            ax.plot(X, T(n, X), c=color)

    widgets.interact(plot_polynom, n=widgets.IntSlider(value=degree, min=start, max=(stop-1), step=step))

